import tkinter as tk
import tkinter.font as font
import tkinter.ttk as ttk
import _tkinter
import time
import os
import numpy as np
import threading
import queue
from multiprocessing import Process, Manager

from muvimaker import main_logger, mv_scratch_key, main_queue
from muvimaker.editor.dialogues import ScratchDirDialogue
from muvimaker.editor.menubar import Menubar
from muvimaker.editor.frames import InfoFrame, LoggingFrame, ParametersFrame, AnalyzerFrame
from muvimaker.core.pictures import PictureError


logger = main_logger.getChild(__name__)


class Editor(tk.Frame):
    def __init__(self, parent):
        logger.debug('setting up editor')
        self.screen_width = parent.winfo_screenwidth()
        self.screen_height = parent.winfo_screenheight()
        tk.Frame.__init__(self, parent, height=self.screen_height, width=self.screen_width, bg='darkgrey')

        self.name = 'editor'
        self.parent = parent
        self.widgets = dict()
        self.master.title('Editor')

        self.progress_bar_queue = queue.Queue()
        self.analyzer_frame_queue = queue.Queue()

        # self._soundfile = tk.StringVar()
        self._pictures = dict()
        self._soundfiles = dict()
        self._videofile = tk.StringVar()
        self._project_handler = None
        self._ongoing_progress = None
        self._progress_bar_thread = None
        self._analysing_thread = None
        self._make_video_thread = None

        self.active = tk.StringVar()
        self.active.set('disabled')
        self.active.trace('w', lambda *_: self._set_widgets_attribute('state', self.active.get()))

        self._setup_widgets()

        self.parent.columnconfigure(1, weight=1)
        self.parent.columnconfigure(0, weight=10)
        self.parent.rowconfigure((0, 1, 2, 3, 4, 5), weight=1)
        self.check_scratch()

    # ------------------    properties   --------------------- #

    @property
    def videofile(self):
        return self._videofile

    @videofile.setter
    def videofile(self, value):
        self._videofile.set(value)

    @property
    def project_handler(self):
        return self._project_handler

    @project_handler.setter
    def project_handler(self, value):
        logger.debug('got ProjectHandler')
        self._project_handler = value
        # self._soundfile.set(str(value.sound_filename))

        self.widgets['info_frame'].sound_files_frame._info_dict = value.sound_files
        self.widgets['info_frame'].sound_files_frame.update_listbox()
        self.widgets['info_frame'].pictures_frame._info_dict = value.pictures
        self.widgets['info_frame'].pictures_frame.update_listbox()

        self.active.set('normal')
        logger.debug("activated")
        if not isinstance(value.analyzer_results, type(None)):
            logger.debug(f'passing saved analyzer results to analyzer frame')
            self.widgets['analyzer_frame'].c_s_lrvf = value.analyzer_results

    # --------------------   Widgets   ------------------------ #

    def add_widget(self, name, widget):
        if name not in self.widgets:
            self.widgets[name] = widget
        else:
            logger.warning(f'{name} already exists! Not adding widget!')

    def _set_widgets_attribute(self, key, state):
        for n, w in self.widgets.items():
            try:
                w[key] = state
            except _tkinter.TclError as e:
                logger.debug(f"{n} does not have option {key}: {e}")

    def _setup_widgets(self):
        menubar = Menubar(self)
        self.add_widget('menu', menubar)
        self.master.config(menu=menubar)

        infoframe = InfoFrame(self)
        self.add_widget('info_frame', infoframe)
        infoframe.grid(row=0, column=0, columnspan=2, sticky='nsew')

        logging_frame = LoggingFrame(self.master)
        self.add_widget('logging_frame', logging_frame)
        logging_frame.grid(row=4, column=0, columnspan=2, sticky='nsew')

        parameters_frame = ParametersFrame(self.master, width=10)
        self.add_widget('parameters_frame', parameters_frame)
        parameters_frame.grid(row=1, column=1, sticky='nsew')

        analyzer_frame = AnalyzerFrame(self)
        self.add_widget('analyzer_frame', analyzer_frame)
        analyzer_frame.grid(row=1, column=0, columnspan=1, rowspan=3, sticky='nsew')

        progress_bar = ttk.Progressbar(self.master, orient='horizontal', mode='indeterminate', length=100)
        progress_bar.grid(row=5, column=0, columnspan=2, sticky='nsew')
        self.add_widget('progress_bar', progress_bar)

        analyse_button = tk.Button(self.master, text='analyse',
                                   height=5, width=10,
                                   state=self.active.get(),
                                   command=self.analyse)
        analyse_button.grid(row=2, column=1)#, sticky='nsew')
        self.add_widget('analyse_button', analyse_button)

        make_video_button = tk.Button(self.master, text='make video',
                                      height=5, width=10,
                                      state=self.active.get(),
                                      command=self.make_video)
        make_video_button['font'] = font.Font(weight='bold')
        make_video_button.grid(row=3, column=1)#, sticky='nsew')
        self.add_widget('make_video_button', make_video_button)

    # ----------------------------------   Functions  ---------------------------------------------- #

    def _start_progress_bar(self):
        self.active.set('disabled')
        self._ongoing_progress = True
        self._progress_bar_thread = threading.Thread(target=self._running_progress_bar)
        self._progress_bar_thread.start()

    def _running_progress_bar(self):
        logger.debug('running progress bar')
        progress_bar = self.widgets['progress_bar']
        progress_bar['value'] = 0
        sgn = 1
        while True:
            try:
                cont = self.progress_bar_queue.get(block=False)
                if not cont:
                    logger.debug('stopping progress bar')
                    break
            except queue.Empty:
                pass

            progress_bar['value'] += 1 * sgn
            if progress_bar['value'] >= 100:
                sgn *= -1
            time.sleep(1 / 10)

    def _stop_progress_bar(self):
        self.active.set('normal')
        self.progress_bar_queue.put(False)
        logger.debug('Put False into queue, waiting for termination')
        self._progress_bar_thread.join()
        self.widgets['progress_bar']['value'] = 0

        # ---------------------------------    ProjectHandler calls    --------------------------------- #

    def make_video(self):
        self.active.set('disabled')
        # threading.Thread(target=self._start_progress_bar).start()
        if not self.project_handler.main_sound_file:
            logger.error(f'No main sound file!')

        else:
            self._start_progress_bar()
            self._make_video_thread = threading.Thread(target=self._make_video)
            self._make_video_thread.start()

    def _make_video(self):
        logger.debug('making video')
        screen_size = np.array(self.widgets['parameters_frame'].size, dtype=float)
        fps = self.widgets['parameters_frame'].fps

        try:
            self.videofile = self.project_handler.make_video(framerate=fps, screen_size=screen_size)
        except PictureError as e:
            logger.error(e)
        finally:
            self._stop_progress_bar()

    def analyse(self):
        self._start_progress_bar()
        self._analysing_thread = threading.Thread(target=self._analyse)
        self._analysing_thread.start()

    def _analyse(self):
        logger.debug('analysing')
        screen_size = np.array(self.widgets['parameters_frame'].size, dtype=float)
        f = 200 / screen_size[0]
        screen_size *= f
        fps = self.widgets['parameters_frame'].fps

        try:
            low_res_video_frames, fps = self.project_handler.analyse(
                screen_size,
                framerate=fps)
            logger.debug('passing results to analyzer frame')
            self.analyzer_frame_queue.put({'c_s_lrvf': (low_res_video_frames, fps)})
            self.widgets['analyzer_frame'].collect()
        except PictureError as e:
            logger.error(e)
        finally:
            self._stop_progress_bar()

    def check_scratch(self):
        scr = os.environ.get(mv_scratch_key, "None")
        if not os.path.isdir(scr):
            logger.debug(f'{scr} is not a directory!')
            ScratchDirDialogue(self)



from .scratchdir_dialogue import ScratchDirDialogue
from .open_project_dialogue import OpenProjectDialogue
from .new_file_dialogue import NewFileDialogue
from .add_sound_dialogue import SoundDialogue
from .add_picture_dialogue import AddPictureDialogue


import tkinter as tk
import numpy as np

from muvimaker import main_logger, get_editor
from muvimaker.editor.dialogues import SoundDialogue


logger = main_logger.getChild(__name__)


class SoundFilesFrame(tk.LabelFrame):

    def __init__(self, parent):
        super(SoundFilesFrame, self).__init__(parent, text='Soundfiles', labelanchor='n')

        self.soundfiles_listbox = None
        self._info_dict = dict()

        self._setup_widgets()
        self.columnconfigure((0, 1), weight=1)
        self.rowconfigure(1, weight=1)

    def _setup_widgets(self):

        # Listbox that will contain the files and names
        soundfiles_list = tk.Listbox(self)
        soundfiles_list.grid(row=0, column=0, columnspan=2, sticky='nsew')
        soundfiles_list.bind('<Double-Button-1>', self._change_file)
        self.soundfiles_listbox = soundfiles_list

        # button to add files
        add_button = tk.Button(self, text='Add', command=self._add_file)
        add_button.grid(row=1, column=0)

        # add button to delete files
        delete_button = tk.Button(self, text='Delete', command=self._delete_file)
        delete_button.grid(row=1, column=1)

    def update_listbox(self):
        self.soundfiles_listbox.delete(0, tk.END)
        for name in self._info_dict.keys():
            self.soundfiles_listbox.insert(tk.END, name)

    @property
    def info(self):
        return self._info_dict

    @info.setter
    def info(self, value):
        name, soundfile = value
        logger.debug(f'got {value}')

        l = self.soundfiles_listbox.get(0, tk.END)
        if name in l:
            ind = np.where(np.array(l) == name)
        else:
            ind = len(l)
            self.soundfiles_listbox.insert(ind, name)

        self._info_dict[name] = soundfile
        get_editor(self).project_handler.sound_files = self._info_dict

    def _add_file(self):
        SoundDialogue(self)

    def _change_file(self, event):
        indice = self.soundfiles_listbox.curselection()
        name = self.soundfiles_listbox.get(indice)
        soundfile = self._info_dict[name]

        self.soundfiles_listbox.delete(indice)
        SoundDialogue(self,
                      def_msg='Change sound file',
                      button_text='Change',
                      soundfile_insert=soundfile,
                      name_insert=name,
                      topmost=True,
                      listbox_indice=indice)

    def _delete_file(self):
        indice = self.soundfiles_listbox.curselection()
        self.soundfiles_listbox.delete(indice)


import os
import tkinter as tk
from tkinter import filedialog as fd
from muvimaker import main_logger


logger = main_logger.getChild(__name__)


class SoundDialogue(tk.Toplevel):
    """dialogue to add sound file and give it a name"""

    def __init__(self, parent, def_msg='Select sound file', button_text='Add',
                 soundfile_insert='enter path', name_insert='enter name',
                 topmost=True, listbox_indice=tk.END):
        tk.Toplevel.__init__(self, parent)

        self.parent = parent
        self.msg = tk.StringVar()
        self.msg.set(def_msg)
        self.button_text = button_text
        self.soundfile_insert = soundfile_insert
        self.name_insert = name_insert
        self.listbox_indice = listbox_indice

        self.soundfile_entry = None
        self.name_entry =None

        self._setup_widgets()
        # self._setup_xtra_widgets()
        self.attributes('-topmost', topmost)
        self.mainloop()

    def _setup_widgets(self):
        msg_label = tk.Label(self, textvariable=self.msg)
        msg_label.pack(fill='both', expand=True)

        soundfile_entry = tk.Entry(self, borderwidth=5)
        soundfile_entry.insert(0, self.soundfile_insert)
        soundfile_entry.pack(fill='both', expand=True)
        self.soundfile_entry = soundfile_entry
        self.soundfile_entry.bind('<1>', self._browse)

        name_entry = tk.Entry(self, borderwidth=5)
        name_entry.insert(0, self.name_insert)
        name_entry.pack(fill='both', expand=True)
        self.name_entry = name_entry

        button = tk.Button(self, text=self.button_text, command=self.button_action)
        button.pack(fill='both', expand=True)

    def _browse(self, event):
        filename = fd.askopenfilename()
        self.soundfile_entry.delete(0, 'end')
        self.soundfile_entry.insert(0, filename)

    def button_action(self):
        soundfile = self.soundfile_entry.get()
        name = self.name_entry.get()

        if os.path.isfile(soundfile):
            logger.debug(f'adding {soundfile}')
            self.parent.info = name, soundfile
            self.destroy()

        else:
            self.msg.set(f'{soundfile} is not a file!')



# class AddSoundDialogue(OpenFileDialogue):
#
#     def __init__(self, parent):
#
#         assert isinstance(parent, tk.Listbox)
#
#         OpenFileDialogue.__init__(self,
#                                   parent,
#                                   title='Add Sound File',
#                                   def_msg='Select sound file',
#                                   button_text='Add',
#                                   insert='enter path')
#
#     def button_action(self):
#         entry = self.entry.get()
#         if os.path.isfile(entry):
#             logger.debug(f'adding {entry}')
#
#             self.destroy()
#
#         else:
#             self.msg.set(f'{entry} is not a file!')


import tkinter as tk
import numpy as np

from muvimaker import main_logger, get_editor
from muvimaker.editor.dialogues import AddPictureDialogue


logger = main_logger.getChild(__name__)


class PicturesFrame(tk.LabelFrame):

    def __init__(self, parent):
        super(PicturesFrame, self).__init__(parent, text='Pictures', labelanchor='n')

        self.pictures_listbox = None
        self._info_dict = dict()

        self._setup_widgets()

        self.columnconfigure((0, 1), weight=10)
        self.columnconfigure(2, weight=1)

    def _setup_widgets(self):

        # Listbox that will contain the files and names
        pictures_list = tk.Listbox(self)
        pictures_list.grid(row=0, column=0, rowspan=2, columnspan=2, sticky='nsew')
        pictures_list.bind('<Double-Button-1>', self._change_file)
        self.pictures_listbox = pictures_list

        # button to add files
        add_button = tk.Button(self, text='Add', command=self._add_file)
        add_button.grid(row=2, column=0)#, sticky='nsew')

        # add button to delete files
        delete_button = tk.Button(self, text='Delete', command=self._delete_file)
        delete_button.grid(row=2, column=1)#, sticky='nsew')

        # add buttons to move the items up and down
        up_button = tk.Button(self, text='UP', command=self._move_item_up)
        up_button.grid(row=0, column=2, sticky='nsew')
        down_button = tk.Button(self, text='DOWN', command=self._move_item_down)
        down_button.grid(row=1, column=2, sticky='nsew')

    def update_listbox(self):
        self.pictures_listbox.delete(0, tk.END)
        for name, info in self._info_dict.items():
            logger.debug(f'inserting {name} to {info[-1]}')
            self.pictures_listbox.insert(info[-1], name)

    def _parse_info_to_ph(self):
        logger.debug(f'parsing {self._info_dict} to ProjectHandler')
        get_editor(self).project_handler.pictures = self._info_dict

    @property
    def info(self):
        return self._info_dict

    @info.setter
    def info(self, value):
        name, picture_file, attributes_list, ind = value
        logger.debug(f'got {value}')

        l = self.pictures_listbox.get(0, tk.END)

        if name in l:
            indice = np.where(np.array(l) == name)[0]
        else:
            indice = np.atleast_1d(ind)[0]
            if indice == 'end':
                indice = len(l)
            self.pictures_listbox.insert(ind, name)

        self._info_dict[name] = [picture_file, attributes_list, str(indice)]
        self._parse_info_to_ph()

    def _add_file(self):
        AddPictureDialogue(self)

    def _change_file(self, event):
        indice = self.pictures_listbox.curselection()
        name = self.pictures_listbox.get(indice)
        pic_file, attributes_list, ind = self._info_dict[name]
        logger.debug(f'{pic_file} {attributes_list}, {indice}')

        if not int(indice[0]) == int(ind):
            raise ValueError(f'indice in listbox {indice[0]} is not indice from info {ind}')

        self.pictures_listbox.delete(indice)
        AddPictureDialogue(self,
                           def_list=attributes_list,
                           def_msg='Change sound file',
                           button_text='Change',
                           soundfile_insert=pic_file,
                           name_insert=name,
                           topmost=True,
                           listbox_indice=indice)

    def _delete_file(self):
        indice = self.pictures_listbox.curselection()
        name = self.pictures_listbox.get(indice)
        new_info = {
            k: v for k, v in self._info_dict.items() if k != name
        }
        self._info_dict = new_info
        self.pictures_listbox.delete(indice)
        self._parse_info_to_ph()

    def _move_item_up(self):
        self._move_item(-1)

    def _move_item_down(self):
        self._move_item(1)

    def _move_item(self, step):
        indice = self.pictures_listbox.curselection()[0]

        if indice <= 0:
            return

        new_indice = indice + step
        entry_to_be_moved = self.pictures_listbox.get(indice)
        logger.debug(f'moving {entry_to_be_moved} from {indice} to {new_indice}')
        self.pictures_listbox.delete(indice)
        self.pictures_listbox.insert(new_indice, entry_to_be_moved)

        # adjust all affected indices
        for name, info in self._info_dict.items():
            iind = int(info[-1])

            if indice < new_indice:
                if (iind > indice) and (iind <= new_indice):
                    self._info_dict[name][-1] = str(iind - 1)

            if indice > new_indice:
                if (iind < indice) and (iind >= new_indice):
                    self._info_dict[name][-1] = str(iind + 1)

        self._info_dict[entry_to_be_moved][-1] = str(new_indice)
        self._parse_info_to_ph()


import tkinter as tk
import numpy as np
from PIL import Image, ImageTk
import time
import threading
from muvimaker import main_logger
from muvimaker.images import baby_file


logger = main_logger.getChild(__name__)


class AnalyzerFrame(tk.LabelFrame):

    def __init__(self, parent):
        logger.debug('creating AnalyzerFrame')
        tk.LabelFrame.__init__(self, parent.master, text='File Info', labelanchor='n')
        self.parent = parent

        self._low_res_video_images = [np.asarray(Image.open(baby_file))]
        self._fps = None
        self._pic_size = (1000, 1000)
        self._img = None
        self._entries = dict()
        self._frame_ind = None
        self._stop = False
        self._queue = parent.analyzer_frame_queue
        self._playing_thread = None

        self.scale = None
        self.preview_label = None

        self._setup_widgets()
        self['state'] = 'disabled'
        self._entries['preview_label']['state'] = 'normal'
        self.columnconfigure((0, 1), weight=1)
        self.rowconfigure((0,1,2,3), weight=1)
        
    def __setitem__(self, key, value):
        if key == 'state':
            for w, entry in self._entries.items():
                if w != 'preview_label':
                    entry['state'] = value
        else:
            raise KeyError(f"\'AnalyzerFrame\' object has no key \'{key}\'")

    @property
    def pic_size(self):
        return self._pic_size

    @property
    def c_s_lrvf(self):
        return self._low_res_video_images, self._fps

    @c_s_lrvf.setter
    def c_s_lrvf(self, tup):
        logger.debug('analyzer frame got results')
        self._low_res_video_images, self._fps = tup
        self.scale.configure(to=len(self._low_res_video_images)-1)
        self.frame_ind = 0

    @property
    def frame_ind(self):
        return self._frame_ind

    @frame_ind.setter
    def frame_ind(self, value):
        self._frame_ind = value
        self._update_preview_label(value)
        # self._update_spectrogram_label(value)

    def _setup_widgets(self):

        # slider to select which frame to display
        self.scale = tk.Scale(self, from_=0, to=1, orient="horizontal",
                              command=self._change_frame)
        self.scale.grid(row=3, column=0, sticky='nsew', columnspan=2)
        self._entries['scale'] = self.scale

        # setting up the label that displays the frame from the low res preview
        logger.debug(f'Ind is {self.scale.get()}')
        preview_label = tk.Label(self, image=self._img)
        preview_label.grid(row=0, column=0, columnspan=2, sticky='nsew')
        self._entries['preview_label'] = preview_label
        self.preview_label = preview_label

        # setting up play button
        play_button = tk.Button(self, text='PLAY', fg='green', command=self.play)
        play_button.grid(row=4, column=0)

        # setting up stop button
        stop_button = tk.Button(self, text='STOP', fg='red', command=self.stop)
        stop_button.grid(row=4, column=1)

    def _get_pil_image_from_array(self, frame_ind, array):
        img_array = array[frame_ind]
        pil_img = Image.fromarray(img_array)
        pil_img.resize(self._pic_size)
        img = ImageTk.PhotoImage(image=pil_img)
        return img

    def _get_image(self, frame_ind):
        if not isinstance(self._low_res_video_images, type(None)):
            return self._get_pil_image_from_array(frame_ind, self._low_res_video_images)
        else:
            logger.warning('No video images analysed!')

    def _update_preview_label(self, frame_ind):
        self._img = self._get_image(frame_ind)
        self.preview_label.configure(image=self._img)

    def _change_frame(self, frame_ind_str):
        self.frame_ind = int(frame_ind_str)

    def play(self):
        self._playing_thread = threading.Thread(target=self._play_thread)
        self._playing_thread.start()

    def _play_thread(self):
        self._stop = False
        logger.debug('play')
        while True:

            logger.debug(f'frame ind is {self._frame_ind}')
            self.scale.set(self._frame_ind + 1)
            self.frame_ind = self._frame_ind + 1
            time.sleep(1 / self._fps)

            if self._frame_ind >= len(self._low_res_video_images) - 1:
                self._stop = True

            if self._stop:
                break

    def stop(self):
        logger.debug('stop')
        self._stop = True
        self._playing_thread.join()
        logger.debug(f"play thread is alive: {self._playing_thread.is_alive()}")

    def collect(self):
        it = self._queue.get()
        if 'c_s_lrvf' in it:
            self.c_s_lrvf = it['c_s_lrvf']
        else:
            logger.error(f"'c_s_lrvf' not in queue item! "
                         f"Item {it} is of type {type(it)}")


import tkinter as tk
from muvimaker import main_logger


logger = main_logger.getChild(__name__)


class ParametersFrame(tk.LabelFrame):

    def __init__(self, parent, **kw):
        tk.LabelFrame.__init__(self, parent.master, text='Parameters', labelanchor='n', **kw)
        self.parent = parent
        self._entries = dict()
        self._setup_widgets()
        self['state'] = 'disabled'

    def __setitem__(self, key, value):
        if key == 'state':
            for entry in self._entries.values():
                entry['state'] = value
        else:
            raise KeyError(f"\'ParameterFrame\' object has no key \'{key}\'")

    @property
    def fps(self):
        return int(self._entries['fps'].get())

    @property
    def size(self):
        return int(self._entries['size_x'].get()), int(self._entries['size_y'].get())

    def _setup_widgets(self):
        width = 5
        # --------   FPS  ------------ #
        fps_row = 0
        fps_label = tk.Label(self, text='FPS: ')
        fps_label.grid(row=fps_row, column=0)

        fps_entry = tk.Entry(self, width=width)
        fps_entry.insert(0, '25')
        fps_entry.grid(row=fps_row, column=1)
        self._entries['fps'] = fps_entry

        # ---------- Size -------------- #
        size_row = 1
        size_label = tk.Label(self, text='Size [px]: ')
        size_label.grid(row=size_row, column=0)

        size_x_entry = tk.Entry(self, width=width)
        size_x_entry.insert(0, '1280')
        size_x_entry.grid(row=size_row, column=1)
        self._entries['size_x'] = size_x_entry

        size_times_label = tk.Label(self, text='x')
        size_times_label.grid(row=size_row, column=2)

        size_y_entry = tk.Entry(self, width=width)
        size_y_entry.insert(0, '720')
        size_y_entry.grid(row=size_row, column=3)
        self._entries['size_y'] = size_y_entry


import tkinter as tk
import os
from muvimaker.editor.dialogues.base_dialogue import BaseDialogue
from muvimaker import main_logger, mv_scratch_key, get_editor
from muvimaker.core.project import ProjectHandler


logger = main_logger.getChild(__name__)


class NewFileDialogue(BaseDialogue):

    def __init__(self, parent):

        BaseDialogue.__init__(self,
                              parent,
                              title='New Project',
                              def_msg='Select name',
                              button_text='Create',
                              insert='enter project name')

    def button_action(self):
        try:
            entry = self.entry.get()
            scr = os.environ[mv_scratch_key]
            project_dir = f'{scr}/{entry}'
            logger.info(f'creating {project_dir}')
            os.mkdir(project_dir)
            editor = get_editor(self)
            editor.project_handler = ProjectHandler(entry, project_dir)
            self.destroy()

        except OSError as e:
            self.msg.set(e)



import os
import tkinter as tk
from tkinter import filedialog as fd

from muvimaker import main_logger
from muvimaker.core.pictures.base_picture import BasePicture


logger = main_logger.getChild(__name__)


class AddPictureDialogue(tk.Toplevel):
    """dialogue to add sound file and give it a name"""

    def __init__(self, picture_frame, def_list=None, def_msg='Select picture file', button_text='Add',
                 soundfile_insert='enter picture class', name_insert='enter name',
                 topmost=True, listbox_indice=tk.END):

        super(AddPictureDialogue, self).__init__()

        self.picture_frame = picture_frame
        self.msg = tk.StringVar()
        self.msg.set(def_msg)
        self.button_text = button_text
        self.picture_insert = soundfile_insert
        self.name_insert = name_insert
        self.listbox_indice = listbox_indice

        try:
            _ = len(def_list)
            logger.debug('No default list')
            self.list = def_list
        except TypeError:
            logger.debug('generating empty list')
            self.list = list()

        self.picture_entry = None
        self.name_entry =None
        self.listbox = None

        self._setup_widgets()
        self.attributes('-topmost', topmost)
        self.mainloop()

    def _setup_widgets(self):
        msg_label = tk.Label(self, textvariable=self.msg)
        msg_label.pack(fill='both', expand=True)

        soundfile_entry = tk.Entry(self, borderwidth=5)
        soundfile_entry.insert(0, self.picture_insert)
        soundfile_entry.pack(fill='both', expand=True)
        self.picture_entry = soundfile_entry

        name_entry = tk.Entry(self, borderwidth=5)
        name_entry.insert(0, self.name_insert)
        name_entry.pack(fill='both', expand=True)
        self.name_entry = name_entry

        listbox = tk.Listbox(self)

        for i, l in enumerate(self.list):
            logger.debug(f"adding {l}")
            listbox.insert(i, l)

        listbox.pack(fill='both', expand=True)
        self.listbox = listbox
        self.listbox.bind('<BackSpace>', self._delete_item)

        item_entry = tk.Entry(self, borderwidth=5)
        item_entry.pack(fill='both', expand=True)
        self.item_entry = item_entry

        add_item_button = tk.Button(self, text='add attribute', command=self._add_attribute)
        add_item_button.pack(fill='both', expand=True)

        button = tk.Button(self, text=self.button_text, command=self.button_action)
        button.pack(fill='both', expand=True)

    def _delete_item(self, event):
        indice = self.listbox.curselection()
        self.listbox.delete(indice)

    def _add_attribute(self):
        self.listbox.insert(tk.END, self.item_entry.get())
        self.item_entry.delete(0, tk.END)

    def button_action(self):
        picture_class = self.picture_entry.get()

        if picture_class in BasePicture.subclasses.keys():
            self.picture_frame.info = self.parse_info()
            self.destroy()

        else:
            self.msg.set(f'{picture_class} not recognised! Available: {BasePicture.subclasses.keys()}')

    def parse_info(self):
        return self.name_entry.get(), self.picture_entry.get(), self.listbox.get(0, tk.END), self.listbox_indice



import tkinter as tk
import argparse

from muvimaker.editor import Editor
from muvimaker import main_logger


logger = main_logger.getChild(__name__)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('--log_level', '-l', type=str, default='DEBUG', const='DEBUG', nargs='?')
    args = parser.parse_args()
    main_logger.setLevel(args.log_level)

    root = tk.Tk(className='MuviMaker')
    screen_width = root.winfo_screenwidth()
    screen_height = root.winfo_screenheight()
    root.geometry(f'{round(screen_width/2)}x{round(screen_height/2)}')
    app = Editor(root)
    root.mainloop()


import os
import logging
import queue

# ======================== #
# ==  customize logger  == #
# ======================== #


class QueueHandler(logging.Handler):
    """Class to send logging records to a queue

    It can be used from different threads
    """

    def __init__(self, queue):
        super().__init__()
        self.log_queue = queue

    def emit(self, record):
        self.log_queue.put(record)


main_logger_name = 'main'
main_logger = logging.getLogger(main_logger_name)
logger_format = logging.Formatter('%(levelname)s - %(name)s: %(message)s')

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(logger_format)
main_logger.addHandler(stream_handler)

main_queue = queue.Queue(-1)
queue_handler = QueueHandler(main_queue)
queue_handler.setFormatter(logger_format)
main_logger.addHandler(queue_handler)

# ====================================== #
# ==  setting up directory structure  == #
# ====================================== #

mv_scratch_key = 'MUVI_SCRATCH'


def set_scratch_dir(directory):
    main_logger.info(f'Setting scratch to {directory}')
    os.environ[mv_scratch_key] = directory


def get_editor(self):
    master = self.master
    while not (hasattr(master, 'name') and (master.name == 'editor')):
        master = master.master
    return master


# ====================================== #
# ==          initialise              == #
# ====================================== #

from muvimaker.core.project import ProjectHandler
from muvimaker.core.video import Video
from muvimaker.core.sound import Sound
from muvimaker.core.pictures import *



import unittest

from muvimaker import main_logger
from muvimaker.core.project import ProjectHandler, standard_screen_size, standard_framerate, standard_hop_length
from muvimaker.example_data import example_song, example_pic
from muvimaker.core.pictures.base_picture import BasePicture


main_logger.setLevel('DEBUG')
logger = main_logger.getChild(__name__)

sounds = {'main': example_song}

pictures = dict()
pictures['bg'] = [
    'background',
    ['colour: 0'],
    '0'
]

j = 1

for pic_class in BasePicture.subclasses.keys():

    if pic_class in ['background', 'pre_computed_picture']:
        continue

    n = f'{j}: {pic_class}'
    pictures[n] = [
        pic_class,
        ['colour: main', 'radius: main'],
        f'{j}'
    ]

    if 'meta' in pic_class:
        pictures[n][1].append('pictures_class: star')

    if 'picture_from_file' in pic_class:
        pictures[n][1].append(f'filename: {example_pic}')
        pictures[n][1].append('scale: 0.01')

    j += 1

ph_name = 'test_ph_handler'
ph = ProjectHandler.get_project_handler(ph_name)
ph.pictures = pictures
ph._main_sound_file = example_song
ph.sound_files = sounds
ph.save_me()


class TestProjectHandler(unittest.TestCase):

    def test_a_project_handler(self):
        logger.info('\n\n   testing project handler\n\n')
        classes = [l[0] for l in pictures.values()]
        logger.debug(f'testing classes {classes}')
        video = ph.get_video(
            standard_screen_size,
            standard_hop_length,
            standard_framerate
        )
        video.make_frame_per_frame(10)

    def test_b_load_project_handler(self):
        logger.info('\n\n    testing a loaded project handler\n\n')
        ph_loaded = ProjectHandler.get_project_handler(ph_name)
        video = ph_loaded.get_video(
            standard_screen_size,
            standard_hop_length,
            standard_framerate
        )
        video.make_frame_per_frame(11)

    def test_c_test_analyse(self):
        logger.info('\n\n   test analyse function \n\n')
        ph.analyse((200, 100), standard_hop_length, 20)

    def test_d_test_make_video(self):
        logger.info('\n\n   testing making video \n\n')
        ph.make_video(screen_size=(100, 100), hop_length=standard_hop_length, framerate=20)


import tkinter as tk
from muvimaker import main_logger, get_editor
from muvimaker.editor.dialogues import OpenProjectDialogue, NewFileDialogue

logger = main_logger.getChild(__name__)


class Filemenu(tk.Menu):

    def __init__(self, menubar):
        logger.debug('setting up filemenu')
        tk.Menu.__init__(self, menubar)
        self._setup_commands()

    def _setup_commands(self):
        logger.debug('setting up commands')
        self.add_command(label='New Project', command=self._new_project)
        self.add_command(label='Open Project', command=self._open_project)
        self.add_command(label='Save Project', command=self._save_project)

    def _new_project(self):
        NewFileDialogue(self)

    def _open_project(self):
        OpenProjectDialogue(self)

    def _save_project(self):
        editor = get_editor(self)
        editor.project_handler.save_me()


import tkinter as tk
from muvimaker import main_logger

from muvimaker.editor.menubar.filemenu import Filemenu
# from muvimaker.editor.menubar.soundmenu import Soundmenu


logger = main_logger.getChild(__name__)


class Menubar(tk.Menu):

    def __init__(self, parent):
        logger.debug('setting up menubar')
        tk.Menu.__init__(self, parent)
        self._setup_cascades()

    def _setup_cascades(self):
        self.add_cascade(label='File', menu=Filemenu(self))
        # self.add_cascade(label='Audio', menu=Soundmenu(self))


from muvimaker import main_logger, mv_scratch_key, set_scratch_dir
from muvimaker.editor.dialogues.base_dialogue import OpenDirectoryDialoge
from tkinter import filedialog as fd
import os


logger = main_logger.getChild(__name__)


class ScratchDirDialogue(OpenDirectoryDialoge):
    """Only to be called when the scratch directory doesn't exist or wasn't given!"""

    def __init__(self, parent):
        logger.debug(f'Calling ScratchDirDialogue')

        mv_scratch = os.environ.get(mv_scratch_key, 'None')
        msg = ''
        if not os.path.isdir(mv_scratch):
            if isinstance(mv_scratch, type(None)):
                msg = 'No working directory given!'
            if isinstance(mv_scratch, str):
                msg = f'{mv_scratch} is not a directory!'
        else:
            logger.warning(f'Called ScratchDirDialogue but {mv_scratch} is a directory!')

        OpenDirectoryDialoge.__init__(self,
                                      parent,
                                      title='select working directory',
                                      def_msg=msg,
                                      button_text='OK',
                                      insert="Enter working directory")

    def button_action(self):
        scr = self.entry.get()
        if os.path.isdir(scr):
            self.set_scratch_and_exit()
        else:
            self.msg.set(f'\"{scr}\" is not a directory!')
            
    def browse(self, event):
        filename = fd.askdirectory()
        self.entry.delete(0, 'end')
        self.entry.insert(0, filename)

    def set_scratch_and_exit(self):
        """Sets the scratch directory to input"""
        set_scratch_dir(self.entry.get())
        self.destroy()


from muvimaker import main_logger
import tkinter as tk
from tkinter import filedialog as fd


logger = main_logger.getChild(__name__)


class BaseDialogue(tk.Toplevel):
    """base class for dialogues"""

    def __init__(self, parent, title, def_msg, button_text, insert='', topmost=True):
        tk.Toplevel.__init__(self, parent)

        self.parent = parent
        self.title = title
        self.msg = tk.StringVar()
        self.msg.set(def_msg)
        self.button_text = button_text
        self.insert = insert

        self.entry = None

        self._setup_standard_widgets()
        self._setup_xtra_widgets()
        self.attributes('-topmost', topmost)
        self.start_mainloop()

    def start_mainloop(self):
        self.mainloop()

    def _setup_standard_widgets(self):
        msg_label = tk.Label(self, textvariable=self.msg)
        msg_label.pack(fill='both', expand=True)

        e = tk.Entry(self, borderwidth=5)
        e.insert(0, self.insert)
        e.pack(fill='both', expand=True)
        self.entry = e

        button = tk.Button(self, text=self.button_text, command=self.button_action)
        button.pack(fill='both', expand=True)

    def _setup_xtra_widgets(self):
        pass

    def button_action(self):
        raise NotImplementedError('To be implemented in subclasses!')


class BrowseDialogue(BaseDialogue):

    def start_mainloop(self):
        self.entry.bind('<1>', self.browse)
        self.mainloop()

    def browse_function(self):
        raise NotImplementedError('To be implemented in subclasses')

    def browse(self, event):
        filename = self.browse_function()
        self.entry.delete(0, 'end')
        self.entry.insert(0, filename)

    def button_action(self):
        raise NotImplementedError('To be implemented in subclasses')


class OpenFileDialogue(BrowseDialogue):
    def browse_function(self):
        return fd.askopenfilename()


class OpenDirectoryDialoge(BrowseDialogue):
    def browse_function(self):
        return  fd.askdirectory()


import tkinter as tk
from tkinter import scrolledtext as st
import logging
import queue
from muvimaker import main_logger, main_queue, logger_format


logger = main_logger.getChild(__name__)


class ConsoleUi:
    """Poll messages from a logging queue and display them in a scrolled text widget"""

    def __init__(self, frame, log_queue=main_queue):

        self.frame = frame

        # Create a ScrolledText wdiget
        self.scrolled_text = st.ScrolledText(frame, state='disabled', height=7)
        self.scrolled_text.grid(row=0, column=0, sticky='nsew')
        self.scrolled_text.configure(font=('TkFixedFont', 10))
        self.scrolled_text.tag_config('INFO', foreground='black')
        self.scrolled_text.tag_config('DEBUG', foreground='gray')
        self.scrolled_text.tag_config('WARNING', foreground='orange')
        self.scrolled_text.tag_config('ERROR', foreground='red')
        self.scrolled_text.tag_config('CRITICAL', foreground='red', underline=1)

        # Create a logging handler using a queue
        self.log_queue = log_queue

        # Start polling messages from the queue
        self.frame.after(100, self.poll_log_queue)

    def display(self, record):
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(logger_format)
        msg = stream_handler.format(record)
        self.scrolled_text.configure(state='normal')
        self.scrolled_text.insert(tk.END, msg + '\n', record.levelname)
        self.scrolled_text.configure(state='disabled')
        # Autoscroll to the bottom
        self.scrolled_text.yview(tk.END)

    def poll_log_queue(self):
        # Check every 100ms if there is a new message in the queue to display
        while True:
            try:
                record = self.log_queue.get(block=False)
            except queue.Empty:
                break
            else:
                self.display(record)
        self.frame.after(100, self.poll_log_queue)


class LoggingFrame(tk.Frame):

    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        ConsoleUi(self)



import tkinter as tk
from muvimaker import main_logger, mv_scratch_key

logger = main_logger.getChild(__name__)


class Soundmenu(tk.Menu):

    def __init__(self, menubar):
        logger.debug('setting up soundmenu')
        tk.Menu.__init__(self, menubar)
        self._setup_commands()
        self.commands = dict()

    def _setup_commands(self):
        logger.debug('setting up commands')
        self.add_command(label='Add Sound', command=self._add_sound)

    def _add_sound(self):
        AddSoundDialogue(self)

    # def get_project_handler(self):
    #     return self.master.get_project_handler()


import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

if __name__ == '__main__':
    setuptools.setup(
        name="muvimaker",
        version="0.2.7",
        author="Jannis Necker",
        author_email="jannis.necker@gmail.com",
        description="A small package to generate moving pictures from sound",
        long_description=long_description,
        long_description_content_type="text/markdown",
        license="MIT",
        keywords="music video picture generation",
        url="https://github.com/JannisNe/muvimaker",
        project_urls={
            "Bug Tracker": "https://github.com/JannisNe/muvimaker/issues",
        },
        packages=setuptools.find_packages(),
        classifiers=[
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 3.9",
        ],
        python_requires='>=3.9',
        install_requires=[
            "moviepy==1.0.3",
            "librosa==0.8.1",
            "numpy==1.20.3",
            "matplotlib==3.4.2",
            "jupyterlab==3.0.16",
            "pillow==8.2.0",
            "gizeh==0.1.11",
            "ffmpeg==1.4",
            "coveralls",
            "setuptools==57.0.0",
        ],
        package_data={'muvimaker': [
            'example_data/example_pic.jpg',
            'example_data/example_song.mp3'
        ]},
        include_package_data=True
    )


import tkinter as tk
from muvimaker import main_logger
from .sound_files_frame import SoundFilesFrame
from .picture_frame import PicturesFrame


logger = main_logger.getChild(__name__)


class InfoFrame(tk.Frame):

    def __init__(self, parent):
        tk.Frame.__init__(self, parent.master)
        self.parent = parent
        self.master = parent
        self.sound_files_frame = None
        self.pictures_frame = None
        self._setup_widgets()
        self.columnconfigure((0, 1), weight=1)
        self.rowconfigure(0, weight=1)

    def _setup_widgets(self):
        sound_files_frame = SoundFilesFrame(self)
        sound_files_frame.grid(row=0, column=0, sticky='nsew')
        self.sound_files_frame = sound_files_frame

        pictures_frame = PicturesFrame(self)
        pictures_frame.grid(row=0, column=1, sticky='nsew')
        self.pictures_frame = pictures_frame

        # audio_file_label = tk.Label(self, text='Sound File: ')
        # audio_file_label.grid(row=0, column=0)
        # audio_filename_label = tk.Label(self, textvariable=self.parent.soundfile)
        # audio_filename_label.grid(row=0, column=1)
        #
        # video_file_label = tk.Label(self, text='Video Output File: ')
        # video_file_label.grid(row=1, column=0)
        # video_filename_label = tk.Label(self, textvariable=self.parent.videofile)
        # video_filename_label.grid(row=1, column=1)


import tkinter as tk
import os
import pickle
from muvimaker.editor.dialogues.base_dialogue import OpenDirectoryDialoge
from muvimaker import main_logger, mv_scratch_key, get_editor
from muvimaker.core.project import ProjectHandler


logger = main_logger.getChild(__name__)


class OpenProjectDialogue(OpenDirectoryDialoge):

    def __init__(self, parent):

        super().__init__(parent,
                         title='Open Project',
                         def_msg='select project',
                         button_text='Open',
                         insert='enter project folder path')

    def _setup_xtra_widgets(self):
        frame = tk.LabelFrame(self, text='Projects in working directory', labelanchor='n')
        frame.pack(fill='both', expand=True)

        scr = os.environ[mv_scratch_key]
        directories_in_scr = os.listdir(scr)

        def ba(d):
            self.entry.delete(0, 'end')
            self.entry.insert(0, f'{scr}/{d}')

        for d in directories_in_scr:
            if '.' in d:
                continue
            button = tk.Button(frame, text=d, command=lambda s=d: ba(s))
            button.pack(fill='both', expand=True)

    def button_action(self):
        entry = self.entry.get()
        if os.path.isdir(entry):
            logger.info(f'opening {entry}.')
            editor = get_editor(self)
            editor.project_handler = ProjectHandler.get_project_handler(directory=entry)
            self.destroy()
        else:
            self.msg.set(f'{entry} is not a file!')


import unittest

from muvimaker import main_logger
from muvimaker.core.pictures import BasePicture
from muvimaker.core.project import ProjectHandler, standard_screen_size, standard_framerate, standard_hop_length
from muvimaker.core.video import Video
from muvimaker.example_data import example_song


logger = main_logger.getChild(__name__)
ph_name = 'test_ph_handler'


class TestPreComputedPicture(unittest.TestCase):

    def test_a_picture(self):
        logger.info('\n\n    test PreComputedPicture\n\n')
        ph = ProjectHandler.get_project_handler(ph_name)
        video = ph.get_video(
            standard_screen_size,
            standard_hop_length,
            standard_framerate
        )
        frames = [video.make_frame_per_frame(10)]

        pre_computed_picture = BasePicture.create(
            'pre_computed_picture',
            None,
            {'frames': frames},
            None
        )

        analyser_video = Video(
            [pre_computed_picture],
            ph.main_sound_file,
            standard_framerate,
            ph.length,
            standard_screen_size
        )

        analyser_video.make_frame_per_frame(0)